# Configure path for the Fast's Agent System Timer
# Inspired from the GSL one, Christopher R. Gabriel <cgabriel@linux.it>, April 2000
# Modified by Martin Quinson <martin.quinson@ens-lyon.fr>, Janv 2002

AC_DEFUN([AM_PATH_FAST],
[
AC_ARG_WITH(FAST,
            AC_HELP_STRING([--with-FAST=DIR],
                           [installation root directory for FAST (optional)]),
            fast_prefix="$withval", fast_prefix="")
AC_ARG_WITH(FAST-bin,
            AC_HELP_STRING([--with-FAST-bin=DIR],
    	                   [installation directory for fast-config (optional)]),
            fast_exec_prefix="$withval", fast_exec_prefix="")

  if test "x$with_FAST" = "xno" ; then
    ifelse([$3], , AC_MSG_WARN([FAST disabled]), [$3])

  else
    if test "x$fast_prefix" != x ; then
      FAST_CONFIG="$fast_prefix/bin/fast-config"
    fi
    if test "x$fast_exec_prefix" != x ; then
      FAST_CONFIG="$fast_exec_prefix/fast-config"
    fi
    if test "x$FAST_CONFIG" = "x" ; then
      AC_PATH_PROG(FAST_CONFIG, fast-config, no)
    fi
    no_fast=""
    if test "$FAST_CONFIG" = "no" ; then
      no_fast=yes
    else
      min_fast_version=$1
      AC_MSG_CHECKING(for FAST version >= $min_fast_version)
      dnl complete the version to match the format %d.%d.%d
      version_length="`echo $min_fast_version|sed 's/\./ /g'|wc -w`"
      if test "`echo $version_length`" = "1" ; then
	min_fast_version=$min_fast_version.0.0
      else if test "`echo $version_length`" = "2" ; then
	min_fast_version=$min_fast_version.0
      fi
      fi
      CFLAGS_FAST=`$FAST_CONFIG --cflags fast`
      CCFLAGS_FAST=`$FAST_CONFIG --cflags fast`
      CXXFLAGS_FAST=`$FAST_CONFIG --cflags fast`
      CPPFLAGS_FAST=`$FAST_CONFIG --cflags fast`
      LIBS_FAST=`$FAST_CONFIG --libs fast`
      fast_complete_version=`$FAST_CONFIG --version|cut -f3 -d' '|sed 's/,$//g'`
      fast_major_version=`echo $fast_complete_version|sed 's/^\([[0-9]]*\).*/\1/'`
      if test "x${fast_major_version}" = "x" ; then
        fast_major_version=0
      fi
      fast_minor_version=`echo $fast_complete_version|\
                          sed 's/^\([[0-9]]*\)\.\{0,1\}\([[0-9]]*\).*/\2/'`
      if test "x${fast_minor_version}" = "x" ; then
        fast_minor_version=0
      fi
      fast_micro_version=`echo $fast_complete_version|\
                          sed 's/^\([[0-9]]*\)\.\{0,1\}\([[0-9]]*\)\.\{0,1\}\([[0-9]]*\).*/\3/'`
      if test "x${fast_micro_version}" = "x" ; then
        fast_micro_version=0
      fi
      ac_save_CFLAGS="$CFLAGS"
      ac_save_CCFLAGS="$CCFLAGS"
      ac_save_CPPFLAGS="$CPPFLAGS"
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CCFLAGS="$CCFLAGS $CCFLAGS_FAST"
      CPPFLAGS="$CPPFLAGS $CPPFLAGS_FAST"
      CXXFLAGS="$CXXFLAGS $CXXFLAGS_FAST"
      LIBS="$LIBS $LIBS_FAST"

      rm -f conf.fasttest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char* my_strdup (const char *str);

char*
my_strdup (const char *str)
{
  char* new_str;
  if (str) {
    new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
    strcpy (new_str, str);
  } else
    new_str = NULL;
  return new_str;
}

int main (void)
{
  int major, minor, micro;
  int n;
  char* tmp_version;

  system ("touch conf.fasttest");
  /* HP/UX 9 writes to sscanf strings */
  tmp_version = my_strdup("$min_fast_version");
  n = sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) ;
  if (n != 2 && n != 3) {
    printf("%s, bad version string\n", "$min_fast_version");
    exit(1);
  }
  if ($fast_major_version > major)
    exit(0);
  if ($fast_major_version < major)
    exit(1);
  if ($fast_minor_version > minor)
    exit(0);
  if ($fast_minor_version < minor)
    exit(1);
  if ($fast_micro_version >= micro)
    exit(0);
  printf("\n*** 'fast-config --version' returned %d.%d.%d, but the minimum version\n",
         $fast_major_version, $fast_minor_version, $fast_micro_version);
  printf("*** of FAST required is %d.%d.%d. If fast-config is correct, then it is\n",
	 major, minor, micro);
  printf("*** best to upgrade to the required version.\n");
  printf("*** If fast-config was wrong, set the environment variable FAST_CONFIG\n");
  printf("*** to point at the correct copy of fast-config, and remove the file\n");
  printf("*** config.cache before re-running configure\n");
  exit(1);
}
],    ,no_fast=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])

      CFLAGS="$ac_save_CFLAGS"
      CCFLAGS="$ac_save_CCFLAGS"
      CXXFLAGS="$ac_save_CXXFLAGS"
      CPPFLAGS="$ac_save_CPPFLAGS"
      LIBS="$ac_save_LIBS"

    fi # if test "$FAST_CONFIG" = "no" ; then

    if test "x$no_fast" = x ; then
      AC_MSG_RESULT(yes)
      ifelse([$2], , :, [$2])     
    else
      AC_MSG_RESULT(no)
      if test "$FAST_CONFIG" = "no" ; then
        echo "*** The fast-config script installed by FAST could not be found"
        echo "*** If FAST was installed in PREFIX, make sure PREFIX/bin is in"
        echo "*** your path, or set the FAST_CONFIG environment variable to the"
        echo "*** full path to fast-config, or use --with-FAST-bin for that."
      else
        if test -f conf.fasttest ; then
          :
        else
          echo "*** Could not run FAST test program, checking why..."
          CFLAGS="$CFLAGS $CFLAGS_FAST"
          CCFLAGS="$CCFLAGS $CCFLAGS_FAST"
          CPPFLAGS="$CPPFLAGS $CPPFLAGS_FAST"
          CXXFLAGS="$CXXFLAGS $CXXFLAGS_FAST"
          LIBS="$LIBS $LIBS_FAST"
          AC_TRY_LINK([
#include <stdio.h>
],        [ return 0; ],
          [echo "*** The test program compiled, but did not run. This usually means"
           echo "*** that the run-time linker is not finding FAST or finding the wrong"
           echo "*** version of FAST. If it is not finding FAST, you'll need to set your"
           echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
           echo "*** to the installed location  Also, make sure you have run ldconfig if that"
           echo "*** is required on your system"
	   echo "***"
           echo "*** If you have an old version installed, it is best to remove it, although"
           echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
          [echo "*** The test program failed to compile or link. See the file config.log for the"
           echo "*** exact error that occured. This usually means FAST was incorrectly installed"
           echo "*** or that you have moved FAST since it was installed. In the latter case, you"
           echo "*** may want to edit the fast-config script: $FAST_CONFIG" ])
           CFLAGS="$ac_save_CFLAGS"
           CCFLAGS="$ac_save_CCFLAGS"
           CPPFLAGS="$ac_save_CPPFLAGS"
           CXXFLAGS="$ac_save_CXXFLAGS"
           LIBS="$ac_save_LIBS"

        fi # if test -f conf.fasttest ; then
      fi # if test "$FAST_CONFIG" = "no" ; then

      ifelse([$3], , AC_MSG_ERROR([Fast not found]), [$3])

    fi # if test "x$no_fast" = x ; then

  fi # if test "x$with_FAST" = "xno" ; then

  AC_SUBST([aci_module_desc_FAST],"performance prediction in a metacomputing environment")
  aci_packages="$aci_packages FAST"
  aci_pkg_config_FAST="fast-config"

  AC_SUBST(CFLAGS_FAST)
  AC_SUBST(CCFLAGS_FAST)
  AC_SUBST(CXXFLAGS_FAST)
  AC_SUBST(CPPFLAGS_FAST)
  AC_SUBST(LIBS_FAST)
  
  rm -f conf.fasttest
  
])

