/* $Id: argset.h,v 1.2 2003/06/03 12:10:59 mquinson Exp $ */

/* argset.c - defines a set of arguments of tests.                           */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                         */
/* Code GPL'ed (see www.gnu.org).                                            */
/* Author(s): Martin Quinson                                                 */

#ifndef _ARGSET_H
#define _ARGSET_H

#ifndef FAST_SEP
#define FAST_SEP '!'
#endif
#if FAST_SEP != '!'
#error FAST_SEP != '!'
#endif

/* internal representation of a set of args */
typedef struct fast_argset_ fast_argset_t;

/* The possible types of an argument */
#define fast_argset_type_count 3
typedef enum {
  fast_argset_all=0,
  fast_argset_num=1,
  fast_argset_string=2
} fast_argset_type_t;

/*===========================================================================*/
/*====[ Creator, destructor ]================================================*/

/*----[ fast_argset_alloc ]--------------------------------------------------*/
/* Alloc the struct and do some initialization                               */
/*---------------------------------------------------------------------------*/
int fast_argset_alloc(const char *binary,
		      /*OUT*/fast_argset_t **where);

/*----[ fast_argset_free ]---------------------------------------------------*/
void fast_argset_free(fast_argset_t *argset);

/*----[ fast_argset_from_string ]--------------------------------------------*/
/* build the argset to bench #binary# described in #string#, frees #string#  */
/*---------------------------------------------------------------------------*/
int fast_argset_from_string(const char *binary,char *string,
			    /*OUT*/fast_argset_t **where);

/*----[ fast_argset_dump ]---------------------------------------------------*/
/* debugging output, always return 1.                                        */
/*---------------------------------------------------------------------------*/
int fast_argset_dump(fast_argset_t *argset);

/*===========================================================================*/
/*====[ to get all possibility ]=============================================*/

/*----[ fast_argset_first ]--------------------------------------------------*/
/* allocate the char** to be passed to execv, reset #argset#, and put the    */
/*  result in the output list.                                               */
/* Returns if all went ok                                                    */
/*---------------------------------------------------------------------------*/
int fast_argset_first(fast_argset_t *argset,
		      /*OUT*/char ***where);

/*----[ fast_argset_next ]---------------------------------------------------*/
/* Increments the list of args (builded by test_args_first) to the next.     */
/* returns true if it was ok, and false if there is no new test to do        */
/*                                                                           */
/* You can retry to increment an argset you know it's done, no segfault.     */
/* If no increment was possible, frees # *args_execv #.                      */
/*---------------------------------------------------------------------------*/
int fast_argset_next(fast_argset_t *argset,char ***args_execv);

/*===========================================================================*/
/*====[ get some stats about the argset ]====================================*/

/*++++[ Stats about the argset (cached) ]++++++++++++++++++++++++++++++++++++*/

/*----[ fast_argset_elem_count ]---------------------------------------------*/
/* Counts how many parameters of type #type# there is                        */
/*  Search for all types if fast_argset_all is given                         */
/* Returns if anything went ok                                               */
/*---------------------------------------------------------------------------*/
int fast_argset_elem_count(fast_argset_type_t type,fast_argset_t *argset,
			   /*OUT*/int *val);

/*----[ fast_argset_possibilities ]------------------------------------------*/
/* Count how many test there will be while variating args of type #type#.    */
/*  Search for all types if fast_argset_all is given                         */
/*  (usefull to allocate a table for each one)                               */
/*---------------------------------------------------------------------------*/
int fast_argset_possibilities(fast_argset_type_t type,fast_argset_t *argset,
			      /*OUT*/int *val);

/*----[ fast_argset_string_id_list ]-----------------------------------------*/
/* Give me the list of possible ids                                          */
/*---------------------------------------------------------------------------*/
int fast_argset_string_id_list(fast_argset_t *argset, /*OUT*/char ***where);

/*++++[ Stats about this test ]++++++++++++++++++++++++++++++++++++++++++++++*/
/*----[ fast_argset_id ]-----------------------------------------------------*/
/* build the id for tests of type #type#, or all if #fast_argset_all# is used*/
/*   (in order to put it in a DB, for example)                               */
/* If searching for all types, append #prefix#                               */
/* The result of this function should be freed after use                     */
/* Returns if anything went ok                                               */
/*---------------------------------------------------------------------------*/
int fast_argset_id(fast_argset_type_t type,const char *prefix,
		   fast_argset_t *argset,
		   /*OUT*/char **where);

/*----[ fast_argset_string_list ]--------------------------------------------*/
/* Return a list of strings of this test.                                    */
/*---------------------------------------------------------------------------*/
char **fast_argset_string_list(fast_argset_t *argset);

/*----[ fast_argset_num_list ]-----------------------------------------------*/
/* build the id for numbers of this test (example, nb=3, list={128,128,128}) */
/*---------------------------------------------------------------------------*/
int fast_argset_num_list(fast_argset_t *argset,
			 /*OUT*/double **where);
#endif /* _ARGSET_H */

