#include "argset.h"
#include "fast_internals.h"

int main (int argc,char *argv[]) {
  fast_argset_t *argset;
  char **arg_execv,*id,*sid,*nid,**id_list;
  int i,elem_count,poss_count;

  if (argc != 2) {
    fprintf(stderr,
	    "Usage: %s string_to_parse\n"
	    "Example of string:\n   `%s'\n",
	    argv[0],
	    "%!s=dgemm;%s=n|t;%s=n|t;%d=128:512:128;%c=3;%a;%a");
    exit (1);
  }
  //  DirectDiagnostics(DIAGDEBUG, stdout);
  DirectDiagnostics(DIAGINFO, stdout);
  DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stderr);
  DirectDiagnostics(DIAGERROR, stderr);
  DirectDiagnostics(DIAGFATAL, stderr);
  

  if (!fast_argset_from_string("binary",strdup(argv[1]),&argset) ||
      !fast_argset_first(argset,&arg_execv)) 
    exit(1);

  //  fast_argset_dump(argset);

  printf("\nPrint all possibilities\n");fflush(stdout);
  do {  
    for (i=0; arg_execv[i]; i++) {
      printf("%s ",arg_execv[i]);
      fflush(stdout);
    }
    printf("\n");fflush(stdout);

    if (!fast_argset_id(fast_argset_all,"BLAS",argset,&id) ||
	!fast_argset_id(fast_argset_string,NULL,argset,&sid) ||
	!fast_argset_id(fast_argset_num,NULL,argset,&nid))
      exit (1);
    
    printf("  Id=`%s' (`%s' -- `%s')\n",id,sid,nid);fflush(stdout);
    free(id);free(sid);free(nid);
    printf("\n");
  } while (fast_argset_next(argset,&arg_execv));

  if (!fast_argset_elem_count   (fast_argset_all, argset,&elem_count) ||
      !fast_argset_possibilities(fast_argset_all, argset,&poss_count))
    exit (1);
  printf("Nb params=%d (%d possibilities)\n",elem_count,poss_count);

  if (!fast_argset_elem_count   (fast_argset_num, argset,&elem_count) ||
      !fast_argset_possibilities(fast_argset_num, argset,&poss_count))
    exit (1);
  printf("Nb num params=%d (%d possibilities)\n",elem_count,poss_count);

  if (!fast_argset_elem_count   (fast_argset_string, argset,&elem_count) ||
      !fast_argset_possibilities(fast_argset_string, argset,&poss_count))
    exit (1);
  printf("Nb string params=%d (%d possibilities)\n",elem_count,poss_count);

  if (!fast_argset_string_id_list(argset,&id_list))
    exit(1);

  for (i=0;i<poss_count;i++)
    printf("String id[%d]=`%s'\n",i,id_list[i]);

  fast_argset_free(argset);
  return 0;
}


