/* $Id: bench_one.h,v 1.4 2003/06/19 16:40:47 mquinson Exp $ */

/* bench_one - run one of the needed experiments */

#ifndef _BENCH_ONE_H
#define _BENCH_ONE_H 1
#include "autobench.h"

/* port.h -- portability declarations for GNU time */

#if __STDC__
# define PARAMS(x) x
# define PTR void *
#else
# define PARAMS(s) ()
# define PTR char *
#endif

#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#if STDC_HEADERS
#include <limits.h>
#include <stdlib.h>
#else
char *getenv PARAMS((const char *var));
PTR malloc PARAMS((size_t sz));
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

/*****************************************************************************/
/** Type declaration                                                        **/
/*****************************************************************************/
/* internal struct on the resources used by a child process*/
struct fast_resource_use_; 
typedef struct fast_resource_use_ fast_resource_use_t;

/* public struct of results */
struct fast_bench_result_ {
  double time;        /* (user + system) time used (in seconds) */
  size_t size;        /* maximum resident set size   */
};
typedef struct fast_bench_result_ fast_bench_result_t;

/* Note: Here is the definition of struct timeval, as given in gettimeofday(2)
   struct timeval {
      long tv_sec;        // seconds 
      long tv_usec;       // microseconds 
   };
*/

/*****************************************************************************/
/** Prototype of the functions                                              **/
/*****************************************************************************/

/*
 *
 * fast_do_bench:
 *
 * @argv: the things to execv()
 * @time: where to put the timing result
 * @space: where to put the spacing result
 * @Returns: if anything went well
 *
 * Calls the 3 next functions to run a bench, and get all needed infos
 * #mem_trace# =                                                             
 *  FAST_MEM_TRACE_NONE => do not trace the memory (let the child run freely)
 *  FAST_MEM_TRACE_SYSCALL => Trace after each syscall                       
 *  FAST_MEM_TRACE_EACHSTEP => Trace after each step (veeeery long)          
 */

int fast_do_bench(int options,char **argv,
		  /*OUT*/double *benched_time,double *space);

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*!!!      The following functions are more or less internal functions    !!!*/
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/


/*
 *
 * fast_resource_use_start:
 *
 * @Returns: a newly allocated struct to pass to the other functions
 *
 * Prepare to measure a child process.
 */

fast_resource_use_t *fast_resource_use_start(int mem_trace,char **argv);

/*
 *
 * fast_resource_use_next:
 *
 * @Returns: true if the child is done (and false if it was 
 * restarted for further tracing)
 *
 * Look for the next execution step of the child
 * Wait for and fill in data on child process PID.
 */

int fast_resource_use_next(fast_resource_use_t *resp);

/*
 *
 * fast_resource_use_end:
 *
 * Frees the resources occupied to trace the child,
 * and change it to an external struct
 */

fast_bench_result_t *fast_resource_use_end(fast_resource_use_t *resp);

#endif /* _BENCH_ONE_H */
