/* $Id: bench_utils.c,v 1.1 2003/05/26 10:51:53 mquinson Exp $ */
/* bench-utils -- some functions I need in all Bench* module */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h> /* va_args */
#include <string.h> /* strchr */

#include "bench_utils.h"

/*----[ mallocset ]----------------------------------------------------------*/
double *dmallocset(size_t size) {
  double *res=(double *)malloc(sizeof(double)*size);

  if (res == NULL) {
    printf("Malloc error\n");
    ((double*)res)[0]=1; /* segfault */
  }
  memset(res,3,size);
  return res;
}

/*----[ getarg ]-------------------------------------------------------------*/
void getarg(int argc, char *argv[], const char *errmsg, const char *fmt, ...) {
  int num_arg=0;
  va_list pa;
  char *str=(char*)fmt;

  char *ch;
  double *db;
  int *i;
  
  va_start(pa,fmt);

  while ((str=strchr(str,'%'))!= NULL) {    
    str++;
    num_arg++;
    if (num_arg == argc) {
      fprintf(stderr,errmsg);
      fprintf(stderr,"  [waited for %d args, got %d]\n",num_arg,argc-1);
      exit(1);
    }

    switch (str[0]) {
    case '%':
      fprintf(stderr,
	      "internal error: please don't put %% in format of getarg\n");
      exit(2);
      break;

    case 'c':
      ch=va_arg(pa,char*);
      *ch=(argv[num_arg])[0];
      break;

    case 'd':
      i=va_arg(pa,int*);
      *i=atol(argv[num_arg]);
      break;

    case 'l':
      db=va_arg(pa,double*);
      *db=atof(argv[num_arg]);
      break;

    default:
      fprintf(stderr,
	      "internal error: unknown format in getarg(): %%%c.\n",str[0]);
      exit(2);
      break;      
    }
  }
  va_end(pa);

  if (num_arg != argc-1) {
    fprintf(stderr,errmsg);
    fprintf(stderr,"  [waited for %d args, got %d]\n",num_arg,argc-1);
    exit(1);
  }

  /* Print the result. Can't use vprintf since params after fmt are scanf fmt*/
  va_start(pa,fmt);
  str=(char*)fmt;
  while (*str != '\0') {
    switch (*str) {
    case '%':
      str++;
      switch (*str) {
      case 'c':
	ch=va_arg(pa,char*);
	printf("%c",*ch);
	break;
	
      case 'd':
	i=va_arg(pa,int*);
	printf("%d",*i);
	break;
	
      case 'l':
	db=va_arg(pa,double*);
	str++; /* pass %lf */
	printf("%f",*db);
	break;
	
      default:
	printf("%%%c",*str);
	break;      
      }
      break;

    default: /* non-% */
      printf("%c",*str);
      break;      
    }
    str++;
  }
}
