/* $Id: buffer.c,v 1.3 2003/06/19 16:18:38 mquinson Exp $ */

/* buffer - generic interface to the different kind of buffering mecanism */

/* Also implement the dummy buffer type: all operation are noop and report failure */

#include <stdio.h>
#include <stdlib.h> /* malloc, free */

#include "buffer.h"
#include "fast_buffer_plugins.h"

const int fast_buff_plugins_count = 1
#ifdef HAVE_DB2
+1
#endif
#ifdef HAVE_DB41
+1
#endif
;

/* Plugins must be in the same order than the corresponding enum type */
/* Last plugin will be tried first while creation */
const fast_buff_plugin_t fast_buff_plugins[] = {
  {fast_buff_dummy_type,"dummy",&fast_buff_dummy_open,&fast_buff_dummy_close,
   &fast_buff_dummy_retrieve, &fast_buff_dummy_store}
#ifdef HAVE_DB2
  ,{fast_buff_db2_type,"Berkley DB2",&fast_buff_db2_open,&fast_buff_db2_close,
    &fast_buff_db2_retrieve, &fast_buff_db2_store},
#endif
#ifdef HAVE_DB41
  ,{fast_buff_db41_type,"Berkley DB4.1",&fast_buff_db41_open,&fast_buff_db41_close,
    &fast_buff_db41_retrieve, &fast_buff_db41_store}
#endif
};

void fast_buff_open(fast_buff_t **whereto,
		    const char *category,const char *file,const char *path) 
{
  int i;

  *whereto=NULL;
  for (i=fast_buff_plugins_count-1; i>=0; i++) {
    fast_buff_plugins[i].open(whereto,category,file,path);
    if (*whereto) return;
  }
}

void fast_buff_close(fast_buff_t *db) 
{
  fast_buff_plugins[ db->type ].close(db);
}

void fast_buff_retrieve(fast_buff_t *db,
			char *string_id, int nb_params, double *params,
			double *db_time,double *db_space) 
{
  fast_buff_plugins[ db->type ].retrieve(db,string_id,nb_params,params,db_time,db_space);
}

void fast_buff_store(fast_buff_t *db,
		     char *string_id, int nb_params, double *params,
		     double db_time,double db_space) 
{
  fast_buff_plugins[ db->type ].store(db,string_id,nb_params,params,db_time,db_space);
}

