/* $Id: fast_buffer_plugins.h,v 1.1 2003/06/17 09:18:01 mquinson Exp $ */

/* buffer_plugins - interface to all buffering plugins                       */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                         */
/* Code GPL'ed (see www.gnu.org).                                            */
/* Author(s): Martin Quinson                                                 */

#ifndef _FAST_BUFFER_PLUGINS_H_
#define _FAST_BUFFER_PLUGINS_H_

#include "config.h" /* various HAVE_ stuff */
#include "buffer.h"

/* Enum on the type of databases */
typedef enum {
  fast_buff_dummy_type
#ifdef HAVE_DB2
  ,fast_buff_db2_type
#endif 
#ifdef HAVE_DB41
  ,fast_buff_db41_type
#endif
} fast_buff_type_t;

/* A database */
struct fast_buff_s {
  fast_buff_type_t type;
  char *routine;
  void *plugdata;
};

/* A plugin type */
typedef struct {
  fast_buff_type_t id;
  const char *name;
  void (*open)(fast_buff_t**,const char*,const char*,const char*);
  void (*close)(fast_buff_t*);
  void (*retrieve)(fast_buff_t*,const char*,int,double*,double*,double*);
  void (*store)(fast_buff_t*,const char*,int,double*,double,double);
} fast_buff_plugin_t;

/*---[ Prototypes of all plugins ]-------------------------------------------*/
#ifdef HAVE_DB2
void fast_buff_db2_open(fast_buff_t **whereto,
			const char *category,const char *file,const char *path);
void fast_buff_db2_close(fast_buff_t *db);
void fast_buff_db2_retrieve(fast_buff_t*db,
			    const char *stringargs,int nb_params,double*params,
			    /*OUT*/double *db_time,double *db_space);
void fast_buff_db2_store(fast_buff_t*db,
			 const char *stringargs,int nb_params,double*params,
			 /*OUT*/double db_time,double db_space);
#endif 

#ifdef HAVE_DB41
void fast_buff_db41_open(fast_buff_t **whereto,
			 const char *category,const char *file,const char *path);
void fast_buff_db41_close(fast_buff_t *db);
void fast_buff_db41_retrieve(fast_buff_t*db,
			     const char *stringargs,int nb_params,double*params,
			     /*OUT*/double *db_time,double *db_space);
void fast_buff_db41_store(fast_buff_t*db,
			  const char *stringargs,int nb_params,double*params,
			  /*OUT*/double db_time,double db_space);
#endif 
void fast_buff_dummy_open(fast_buff_t **whereto,
			  const char *category,const char *file,const char *path);
void fast_buff_dummy_close(fast_buff_t *db);
void fast_buff_dummy_retrieve(fast_buff_t*db,
			      const char *stringargs,int nb_params,double*params,
			      /*OUT*/double *db_time,double *db_space);
void fast_buff_dummy_store(fast_buff_t*db,
			   const char *stringargs,int nb_params,double*params,
			   /*OUT*/double db_time,double db_space);


/****[ The table of the plugins ]****/
extern const int fast_buff_plugins_count;
extern const fast_buff_plugin_t fast_buff_plugins[];


#endif /* _FAST_BUFFER_PLUGINS_H_ */
