/* $Id: getinfo.h,v 1.2 2003/06/19 16:19:56 mquinson Exp $ */

/* bench_portable - tracing and getting info about child in a portable manner*/

#ifndef BENCH_PORTABLE_H
#define BENCH_PORTABLE_H

#include <sys/types.h>
#include <unistd.h>
#include "../config.h"

/**
 * fast_get_child_sizetime:
 * @pid: The pid of the child you want to get informations about
 * @size: where to store the current measured size of the child
 * @measured_time: where to store the current measured time (user+system elapsed time) of the child
 * @Returns: true if ok, false on error
 * 
 * Get informations about a child process
 */

int fast_get_child_sizetime(pid_t pid,/*OUT*/int *size,double *measured_time);

/*----[ Portable interface to ptrace ]---------------------------------------*/
/* fast_ptrace_traceme: the child must do this in order to be tracable       */
/* fast_ptrace_big_step: the father does to let the child run a big step.    */
/* fast_ptrace_small_step: the father let the child run a small step.        */
/* fast_ptrace_kill: portable call to ptrace the father does kill its child. */
/*---------------------------------------------------------------------------*/
#include <sys/ptrace.h>

/* FAST_PTRACE_TRACEME is defined from configure in bench_utils.h{.in} */

#ifdef HOST_OS_LINUX
# define fast_ptrace_big_step(pid,signal) \
              ptrace(PTRACE_SYSCALL,pid,signal,NULL);
# define fast_ptrace_small_step(pid,signal) \
              ptrace(PTRACE_SINGLESTEP,pid,signal,NULL);
# define fast_ptrace_kill(pid) \
              ptrace(PTRACE_KILL,pid,NULL,NULL);
#endif /* LINUX */


#if defined(HOST_OS_OSF5_1) || defined(HOST_OS_OPENBSD) || defined(HOST_OS_DARWIN)
# define fast_ptrace_big_step(pid,signal) \
              ptrace(PT_CONTINUE,pid,signal,1);
# define fast_ptrace_small_step(pid,signal) \
              ptrace(PT_STEP,pid,signal,1);
# define fast_ptrace_kill(pid) \
              ptrace(PT_KILL,pid,NULL,NULL);
#endif /* OSF5_1 or OPENBSD or DARWIN */

#ifdef HOST_OS_SOLARIS
# define fast_ptrace_big_step(pid,signal) \
              ptrace(7,pid,signal,NULL);
# define fast_ptrace_small_step(pid,signal) \
              ptrace(9,pid,signal,NULL);
# define fast_ptrace_kill(pid) \
              ptrace(8,pid,NULL,NULL);
#endif /* SOLARIS */


#endif /* BENCH_PORTABLE_H */
