/* $Id: getinfo_portable.c,v 1.1 2003/06/19 12:18:50 mquinson Exp $ */

/* bench_portable - tracing and getting info about child in a portable manner*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"

#include "getinfo.h"

#ifdef HOST_OS_LINUX
# include "getinfo_linux.c"
#else /* !LINUX */
# ifdef HOST_OS_SOLARIS
#  include "getinfo_solaris.c"
#else /* !LINUX && !SOLARIS*/

inline char *fast_get_child_resource(pid_t pid,const char *resource);

/* fast_get_child_resource -- return the #resource# of #pid# */
inline char *fast_get_child_resource(pid_t pid,const char *resource) {
  char cmd[256];
  FILE *ps;
  static char res[256];
  
  res[0]='\0';
  sprintf(cmd,"ps -o '%s' -p %d",resource,(int)pid);
  ps=popen(cmd,"r");
  if (ps) {
    fgets(res,sizeof(res),ps); /* skip first line */

    if (!fgets(res,sizeof(res),ps)) /* get the result */
      res[0]='\0';
    pclose(ps);
  }
  return res;
}

int fast_get_child_sizetime(pid_t pid,/*OUT*/int *size,double *mestime) {
  int day=0,hour=0,min=0,sec=0;
  char *p,*res = fast_get_child_resource(pid,"rss,time");

  if (res[0]=='\0') {
    *size=*mestime=0;
    return 0; /* too late, child is dead and freed */
  }

  //  printf("res=%s",res);
  /* Get rss */
  while (*res==' ') res++;
  p=res;
  while (*(++p)!=' ');
  *p='\0';
  *size=atoi(res);

  res=p+1;
  while (*res==' ') res++;

  /* get time */
  // Format on Solaris:  [[dd-]hh:]mm:ss
  // FIXME: I hope it's the same elsewhere :( 

  p=strrchr(res,':');
  if (!p) {
    sec=atoi(res);
    goto fast_get_child_sizetime_done;
  }
  //printf("sec=%s",p+1);
  sec=atoi(p+1);
  *p='\0';

  p=strrchr(res,':');
  if (!p) {
    min=atoi(res);
    goto fast_get_child_sizetime_done;
  }
  //printf("min=%s\n",p+1);
  min=atoi(p+1);
  *p='\0';

  if ((p=strrchr(res,':'))) {
    if (!p) {
      hour=atoi(res);
      goto fast_get_child_sizetime_done;
    }
    printf("hour=%s\n",p+1);
    //hour=atoi(p+1);
    *p='\0';

    if ((p=strrchr(res,'-'))) {
      //printf("day=%s\n",p+1);
      day=atoi(p+1);
    }
  }

 fast_get_child_sizetime_done:
  *mestime= day*86400 + hour*3600 + min*60 + sec;
  *mestime = *mestime ? *mestime : 0.01; /* FIXME: don't return 0, but precision */
  return 1;
}
#endif /* !SOLARIS*/
#endif /* !LINUX */
