/* $Id: getinfo_solaris.c,v 1.1 2003/06/19 16:15:34 mquinson Exp $ */

/* getinfo_solaris -- getting info about child under solaris */

#include "getinfo.h" /* see this file for more info */

#include <procfs.h> /* How to read the Solaris /proc fs */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

int fast_get_child_sizetime(pid_t pid,/*OUT*/int *size,double *time) {
  static pid_t tracedpid=0;
  static int fdstatus=-1,fdusage=-1;
  static char filenamestatus[256],filenameusage[256];
  pstatus_t status;
  prusage_t usage;

  /* init */
  *time = *size  = -1;

  /* If we change pid */
  if (tracedpid != pid) {
    if (fdstatus != -1) {
      close(fdstatus);
      fdstatus=-1;
    }
    if (fdusage != -1) {
      close(fdusage);
      fdusage=-1;
    }
    sprintf(filenamestatus,"/proc/%d/status",pid);
    sprintf(filenameusage,"/proc/%d/usage",pid);
    tracedpid = pid;
  }
  /* read /proc file */
  //  printf("filename=%s;fd=%d;pid=%d;tracedpid=%d\n",filename,fd,pid,tracedpid);
  if (fdstatus == -1 && (fdstatus = open(filenamestatus, O_RDONLY)) == -1) {
    fprintf(stderr, "Error: /proc must be mounted to read %s\n",filenamestatus);
    exit(102);
  }
  if (fdusage == -1 && (fdusage = open(filenameusage, O_RDONLY)) == -1) {
    fprintf(stderr, "Error: /proc must be mounted to read %s\n",filenameusage);
    exit(102);
  }
  if (lseek(fdusage, 0L, SEEK_SET) || lseek(fdstatus, 0L, SEEK_SET)) {
    fprintf(stderr, "Error: cannot seek /proc files\n");
    return 0;
  }
  
  if ((read(fdstatus, &status, sizeof(status))) != sizeof(status)) {
    //      fprintf(stderr, "Error: cannot read %s\n",filenamestatus);
    return 0;
  }
  if ((read(fdusage, &usage, sizeof(usage))) != sizeof(usage)) {
    fprintf(stderr, "Error: cannot read %s\n",filenameusage);
    return 0;
  }
  
  /* parse the status and usage */
  *time = usage.pr_utime.tv_sec + usage.pr_stime.tv_sec +
    usage.pr_utime.tv_nsec/1000000000 + usage.pr_stime.tv_nsec/1000000000;
  *size = status.pr_brksize + status.pr_stksize;
  return 1;
}
