/* $Id: input_text.c,v 1.1 2003/05/26 10:51:53 mquinson Exp $ */

/* input_text - parse a text file to retrive experiments to do */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "input_text.h"

#define BUFFER_SIZE 256

/*====[ prototype of private functions ]=====================================*/
static int fast_get_value(char *buffer,const char *id,char **whereto,
			  int nb_line,char *filename);

/*====[ CODE ]===============================================================*/
/*----[ fast_get_value ]------------------------------------------(private)--*/
/* if #buffer# contains the line defining #id# (ie "#id#=..."), put the val  */
/*  in #whereto# (which should be freed afterward). If the val was already   */
/*  set, outputs an error msg, and return false. If ok, return true.         */
/*---------------------------------------------------------------------------*/
static int fast_get_value(char *buffer,const char *id,char **whereto,
			  int nb_line,char *filename)
{
  char *p;

  if (!strncasecmp(buffer,id,strlen(id))) {
    if (*whereto) {
      fprintf(stderr,
	      "Error while reading %s at line %d: %s is redefined.\n",
	      filename,nb_line,id);
      return 0;
    }
    p=buffer+strlen(id);
    while (*p != '=') p++;
    p++;
    while (*p == ' ' || *p == '\t') p++;
    *whereto=strdup(p);
    //    printf("set %s to %s\n",p,id);
  }
  return 1;
}

/*----[ fast_input_text ]----------------------------------------------------*/
/* parse a text file describing the todos, and add the todo to workspace     */
/*---------------------------------------------------------------------------*/
int fast_bench_from_text(char *filename,fast_bench_workspace_t *ws)
{
  char *category,*binary,*args,*options_str;
  int options;
  char *p,*eof;
  char buffer[BUFFER_SIZE];
  int nb_line=0,found_comment;

  FILE *input;

  if (!(input=fopen(filename,"r"))) {
    fprintf(stderr,
	    "Impossible to open %s to read the list of todo\n",filename);
    return 0;
  }

  category=binary=args=options_str=NULL;

  do {
    options=FAST_DEFAULT_OPTIONS;

    do {
      if ((eof=fgets(buffer,BUFFER_SIZE,input))) {
	nb_line++;
	p=buffer;
	while (*p != '\n' && *p != '\0') p++;
	*p='\0';

	/* remove leading blank chars */
	p=buffer;
	while (*p == ' ' || *p == '\t')
	  p++;
	memmove(p,buffer,p-buffer);
	
	/* remove comments */
	if ((p=strchr(buffer,'#'))) {
	  *p='\0';
	  found_comment=1;
	} else { 
	  found_comment=0;
	}
	
	/* remove ending blank chars */
	p=buffer; 
	while (*p != '\0') p++;
	while (*(--p) == ' ' || *p == '\t') *p = '\0';
	
	/* read the line */
	if (!fast_get_value(buffer,"CATEGORY",&category,nb_line,filename))
	  return 0;
	if (!fast_get_value(buffer,"BINARY",&binary,nb_line,filename))
	  return 0;
	if (!fast_get_value(buffer,"ARGS",&args,nb_line,filename))
	  return 0;
	if (!fast_get_value(buffer,"OPTIONS",&options_str,nb_line,filename))
	  return 0;
      }
    } while (eof != NULL /* not EOF */
	     && (strcmp(buffer,"") || found_comment));


    if (options_str) options=atoi(options_str);
    if (category && binary && args) {
       printf("\ncategory=%s\nbinary=%s\nargs=%s\noptions=%d\n",
	      category,binary,args,options);
      if (!fast_bench_workspace_add(ws,category,binary,args,options)) {
	printf("Arg, I can't add this element to the workspace !\n");
	return 0;
      }
      if (category)    free(category);
      if (binary)      free(binary);
      if (args)        free(args);
      if (options_str) free(options_str);
      category=binary=args=options_str=NULL;
    } /* else {
      printf("One of the args was null. Discarding this element\n");
    }*/
  } while (eof != NULL);

  printf("\n");
  fclose(input);
  return 1;
}
