/* $Id: store2ldif.c,v 1.3 2003/06/14 09:16:28 mquinson Exp $ */

/* store2ldif.c - plugin for export to ldif                                  */
/*                (text file which can easily be added to ldap)              */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                         */
/* Code GPL'ed (see www.gnu.org).                                            */
/* Author(s): Martin Quinson                                                 */

#include "store2ldif.h"
#include "autobench.h"
#include "bench_all.h"

#include <stdlib.h> /* getenv */
#include <stdio.h>
#include <netdb.h> /* gethostbyname */
#include <sys/socket.h> /* inet_ntoa */
#include <netinet/in.h> /* inet_ntoa */
#include <arpa/inet.h>  /* inet_ntoa */
#include <string.h> /* strdup */
#include <unistd.h> /* gethostname */

#define IPaddr_type unsigned int

/*----[ store2txt ]----------------------------------------------------------*/
/*                                                                           */
/* dump the to_store struct. Almost only for debuging                        */
/*                                                                           */
/*---------------------------------------------------------------------------*/
int store2txt(fast_benchs_to_store_t *to_store) 
{
  if(!to_store) {
    printf("The to_store struct is null !\n");
    return 0;
  }
  printf("Dump of the to_store struct :\n");
  printf("  category=(%s)\n",to_store->category);
  printf("  id=(%s)\n",to_store->id);
  printf("  time_interp =(%s)\n",to_store->time_interp);
  printf("  space_interp=(%s)\n",to_store->space_interp);
  return 1;
}


int store2ldif(fast_benchs_to_store_t *to_store,char *name) 
{
  struct hostent *host;
  char buffer[512];
  char *hostname;

  fast_benchs_to_store_t *oneres,*nextres; /* iterator on results */
  char *category,*subcategory;
  char *lastcategory=NULL,*lastsubcategory=NULL;

  FILE *out;

  const char *suffix="ou=Hardware,dc=LIP,dc=ens-lyon,dc=fr";


  if (getenv("FAST_HOSTNAME")) {
    if (!(hostname=malloc(sizeof(char)*
			  (strlen(getenv("FAST_HOSTNAME"))+1)))) {
      fprintf(stderr,"Out of memory\n");
      return 0;
    }
    strcpy(hostname,getenv("FAST_HOSTNAME"));

  } else if (getenv("HOSTNAME")) {
    if (!(hostname=malloc(sizeof(char)*
			  (strlen(getenv("HOSTNAME"))+1)))) {
      fprintf(stderr,"Out of memory\n");
      return 0;
    }
    strcpy(hostname,getenv("HOSTNAME"));
  } else {
    /* get hostname and ip */
    gethostname(buffer,512);
    if ((host = gethostbyname(buffer)) == NULL) {
      fprintf(stderr,"gethostbyname()");
      return 0;
    }
    hostname=strdup(host->h_name);
  } 
  if (!hostname) {
    fprintf(stderr,"%s%s",
	    "fast: The hostname was not given, and I can't manage to\n",
	    " access the name of this host. No output file created\n");
    return 0;
  }
  
  /* get the filename and open it */
  if (!name) {
    sprintf(buffer,"%s.ldif",hostname);
    name=buffer;
  }
  if (!(out=fopen(name,"w"))) {
    fprintf(stderr,
	    "%s:%d: Impossible to open file '%s' to output the result\n",
	    __FILE__,__LINE__,name);
    return 0;
  }

  /* outputs the result */
  fprintf(out,"\
dn: ou=%s,%s\n\
objectclass: top\n\
objectclass: organizationalUnit\n\
ou: %s\n\n",
	  hostname,suffix,
	  hostname);

  oneres=to_store;
  while(oneres) {

    /* get category/subcategory */
    category=strdup(oneres->category);
    subcategory=category;
    while (*subcategory!='/' && *subcategory!='\0') subcategory++;  
    if (*subcategory!='\0') {
      *subcategory='\0';
      subcategory++;
    }
    
    if (!lastcategory || strcmp(lastcategory,category)) {
      if (lastcategory) 
	free(lastcategory);
      fprintf(out,"\
dn: ou=%s,ou=%s,%s\n\
objectclass: top\n\
objectclass: organizationalUnit\n\
ou: %s\n\n",
	      category,hostname,suffix,
	      category);
      lastcategory=category;
    }

    /* if there is no id, no need to make a 'ou' for this test. */
    if (strlen(oneres->id) && (!lastsubcategory || strcmp(lastsubcategory,subcategory))) {
      fprintf(out,"\
dn: ou=%s,ou=%s,ou=%s,%s\n\
objectclass: top\n\
objectclass: organizationalUnit\n\
ou: %s\n\n",
	      subcategory,category,hostname,suffix,
	      subcategory);
      lastsubcategory=subcategory;
    }

    /* FIXME: time=desc space=street */

    fprintf(out,"\
dn: ou=%s%s%s,ou=%s,ou=%s,%s\n\
objectclass: top\n\
objectclass: organizationalUnit\n\
postalAddress: %s\n\
description: %s\n\
street: %s\n\
ou: %s\n\n",
	    ( strcmp(oneres->id,"") ? oneres->id : ""),
	    ( strcmp(oneres->id,"") ? ",ou="    : ""),
	    subcategory,category,hostname,suffix,
	    ( strcmp(oneres->id,"") ? oneres->id : subcategory),
	    oneres->time_interp,
	    oneres->space_interp,
	    ( strcmp(oneres->id,"") ? oneres->id : subcategory));
    
    nextres=oneres->next;
    if (oneres->id)           free(oneres->id);
    if (oneres->category)     free(oneres->category);
    if (oneres->time_interp)  free(oneres->time_interp);
    if (oneres->space_interp) free(oneres->space_interp);
    free(oneres);
    oneres=nextres;
  }
  fclose(out);
  free(hostname);
  return 1;
}
/*
main() {
  fast_benchs_to_store_t to_store;
  to_store.category=strdup("BLAS/dgemm");
  to_store.id=strdup("n!n");
  to_store.time_interp="time_interp";
  to_store.space_interp="space_interp";
  to_ldif(&to_store);
  return 0;
}
*/
