/* $Id: testsset.c,v 1.2 2003/06/03 12:10:59 mquinson Exp $ */

/* testsset.c - functions to handle a set of tests grouped by string id      */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                         */
/* Code available under BSD-like licence (see COPYRIGHT file).               */
/* Author(s): Martin Quinson                                                 */

#include "testsset.h" /* interface of this module */
#include "fast_internals.h"

void fast_bench_testsset_free(fast_bench_testsset_t *testsset)
{
  if (testsset) {
    if (testsset->id)        free(testsset->id);
    if (testsset->t_params)  free(testsset->t_params);
    if (testsset->s_params)  free(testsset->s_params);
    if (testsset->t_val)     free(testsset->t_val);
    if (testsset->s_val)     free(testsset->s_val);
    if (testsset->t_buf)     free(testsset->t_buf);
    if (testsset->s_buf)     free(testsset->s_buf);
    free(testsset);
  }
}

/* creates a testsset */
fast_bench_testsset_t * 
fast_bench_testsset_new(char *id /* id to use */, 
			 int size /* how many bench planed */,
			 /* how many numerical params in benchs */
			 int nb_params)
{
  fast_bench_testsset_t *res;

  if (!(res=(fast_bench_testsset_t *)
	malloc(sizeof(fast_bench_testsset_t)))) {
    ERROR("Malloc error\n");
    return NULL;
  }
  if (!(res->t_params=(double *)
	malloc(sizeof (double) * size * nb_params))) {
    ERROR1("Malloc error: can't allocate %d doubles\n",size * nb_params);
    free(res);
    exit (1);
    return NULL;
  }
  if (!(res->s_params=(double *)
	malloc(sizeof (double) * size * nb_params))) {
    ERROR("Malloc error\n");
    free(res);
    free(res->t_params);
    return NULL;
  }
  if (!(res->t_val=(double *)malloc(sizeof (double) * size))) {
    ERROR("Malloc error\n");
    free(res->t_params);
    free(res->s_params);
    free(res);
    return NULL;
  }
  if (!(res->s_val=(double *)malloc(sizeof (double) * size))) {
    ERROR("Malloc error\n");
    free(res->t_val);
    free(res->t_params);
    free(res->s_params);
    free(res);
    return NULL;
  }
  if (!(res->t_buf=(double *)malloc(sizeof (double) * size))) {
    ERROR("Malloc error\n");
    free(res->s_val);
    free(res->t_val);
    free(res->t_params);
    free(res->s_params);
    free(res);
    return NULL;
  }
  if (!(res->s_buf=(double *)malloc(sizeof (double) * size))) {
    ERROR("Malloc error\n");
    free(res->t_buf);
    free(res->s_val);
    free(res->t_val);
    free(res->t_params);
    free(res->s_params);
    free(res);
    return NULL;
  }
  res->t_size=0;
  res->s_size=0;
  res->id=id;
  res->nb_params=nb_params;
  return res;
}

void fast_bench_testsset_dump(fast_bench_testsset_t *testsset) {
  int i,j;
  int nbp=testsset->nb_params;

  printf ("DUMP THE TESTSSET");
  printf ("\tID=%s\n",testsset->id);
  printf ("   TIME ");
  printf ("\t\tsize=%d\n",testsset->t_size);
  for (i=0;i<testsset->t_size;i++) {  
    printf ("      (");
    for (j=0;j<nbp;j++) 
      printf("%5.5f ",testsset->t_params[nbp*i+j]);
    printf (")  benched: %f ; buffered: %f\n",
	    testsset->t_val[i],testsset->t_buf[i]);
  }
  printf ("   SPACE");
  printf ("\t\tsize=%d\n",testsset->s_size);
  for (i=0;i<testsset->s_size;i++) {  
    printf ("      (");
    for (j=0;j<nbp;j++) 
      printf("%5.5f ",testsset->s_params[nbp*i+j]);
    printf (")  benched: %f ; buffered: %f\n",
	    testsset->s_val[i],testsset->s_buf[i]);
  }
}

/*----[ f_b_w_clean ]--------------------------------------------------------*/
/* Removes empty values from the table before interpolation                  */
/*---------------------------------------------------------------------------*/
int 
fast_bench_testsset_clean(fast_bench_testsset_t *testsset)
{
  /* FIXME: please really do it */
  return 1;
}

