/* $Id: fast_avail_plugins.h,v 1.3 2003/06/04 15:31:21 mquinson Exp $ */

/* fast_avail_plugins - function of sensors plugins (ie, measurements acquisition) */

/* Copyright (C) 2001 the OURAGAN project. Code GPL'ed (see www.gnu.org).    */
/* Author(s): Martin Quinson                                                 */

#ifndef _FAST_AVAIL_PLUGINS_H_
#define _FAST_AVAIL_PLUGINS_H_

#include "fast_internals.h"


/* What is stored in the cache */
typedef struct fast_avail_elm_s {
  int resource;/* FAST_RESOURCE of this cache element */
  char *ID;    /* sensor ID (if needed ; seriename in NWS) */
  int plugin;  /* plugin which created this */

  double ctime; /* timestamp of creation */
  double ttl;   /* time to live (sec) */
  double value;  /* well, ... */

  double lastused; /* last time someone requested this measurement 
                      (to stop autofetching when unused) */
  int autofetched; /* boolean indicating if we're actually autofetching */

  void *plugin_data; /* whatever the plugin wants */
}fast_avail_elm_t;

/* the cache */
extern fast_cache_t *fast_avail_cache[FAST_RESOURCE_TYPE_COUNT];

/* A plugin type */
typedef struct {
  const char *name;
  int (*init)(void);
  int (*elm_init)(int,char**,fast_avail_elm_t*);
  int (*refresh)(fast_avail_elm_t*);
  int (*autofetch_start)(fast_avail_elm_t*);
  int (*autofetch_work)(void);
  int (*edges_list)(int,int*,char***,int*,int**);
  void (*restart_cpu)(const char*);
  void (*data_free)(void*);
} fast_avail_plugin_t;

/****[ Prototypes of the plugins functions ]****/
#ifdef HAVE_NWS
int fast_nws_init(void);
int fast_nws_elm_init(int resource,char **hosts,fast_avail_elm_t *elm);
int fast_nws_refresh(fast_avail_elm_t *elm);
int fast_nws_autofetch_start(fast_avail_elm_t *elm);
int fast_nws_autofetch_work(void);
int fast_nws_edges_list(int resource,
			int *vertex_count, char ***vertex_list,
			int *edges_count, int **edges_list);
void fast_nws_restart_cpu(const char *host);
#endif
int fast_internal_sensor_init(void);
int fast_internal_sensor_elm_init(int resource,char **hosts,fast_avail_elm_t *elm);
int fast_internal_sensor_refresh(fast_avail_elm_t *elm);
void fast_internal_sensor_restart_cpu(const char *host);

int fast_pathfinder_elm_init(int resource,char **hosts,fast_avail_elm_t *elm);
int fast_pathfinder_refresh(fast_avail_elm_t *elm);
void fast_pathfinder_data_free(void *);


/****[ The table of the plugins ]****/
extern const int fast_avail_plugins_count;
extern const fast_avail_plugin_t fast_avail_plugins[];

/*----[ host count for each resource ]---------------------------------------*/
extern int fast_resource_nb_hosts[FAST_RESOURCE_TYPE_COUNT];
/*----[ Cache in which avail_cache elements are stored by ID ]---------------*/
extern fast_cache_t *fast_avail_cache_IDs;

/*----[ Configuration ]------------------------------------------------------*/
extern int fast_config_autofetch;
extern double fast_config_autofetch_on;
extern double fast_config_autofetch_off;
#endif /* _FAST_NEED_PLUGINS_H_ */
