#ifndef INTERP_H
#define INTERP_H
#include <stdio.h>

/*******************
 * Type definition *
 *******************/
struct fast_estimator_; /* internal struct */
typedef struct fast_estimator_ fast_estimator_t;

/* functions to declare a new estimator */
fast_estimator_t *fast_estimator_alloc_empty(void);
fast_estimator_t *fast_estimator_alloc_array(int nbparams,
					     int nbxp,
					     double *xp,double *y,
					     int maxorder);
fast_estimator_t *fast_estimator_alloc_vargs(int nbparams,
					     int nbxp,
					     int maxorder,...);


void fast_estimator_free(fast_estimator_t *estimator);

/* returns the chisquare observed during the fiting */
double fast_estimator_chisq(fast_estimator_t *estimator);

/* functions to use an estimator */
double fast_estimator_use_array(fast_estimator_t *estimator,double *args);
double fast_estimator_use_vargs(fast_estimator_t *estimator,...);

/* function to test the quality of an estimator */
void fast_estimator_test_array(fast_estimator_t *estimator,
			       int nbparams,
			       int nbxp,
			       double *xp,double *y);

/* IO functions */
/* fast_estimator_dump: mainly for debug purpose */
void fast_estimator_dump(FILE *stream,fast_estimator_t *estimator);
/* fast_estimator_to_string: returns the estimator in a newly allocated string
   (to store it in LDAP) */
char *fast_estimator_to_string(fast_estimator_t *estimator);
fast_estimator_t *fast_estimator_alloc_from_string(char *string);
#endif /* INTERP_H */
