/* $Id: fast_config.c,v 1.6 2003/06/06 14:53:07 pmachard Exp $ */

/* fast_config - toolset to manage FAST configuration                     */

/* Copyright (C) 2002 the OURAGAN project. Code GPL'ed (see www.gnu.org). */
/* Author(s): Martin Quinson                                              */

#include "fast_internals.h"

fast_cache_t *fast_config;

/**
 * fast_config_init:
 * @fisrt:
 * @pa:
 * @Returns: True if everything went ok, false if an error occurs
 *
 * Initialize fast config
 */

int fast_config_init(const char *first, va_list pa){
  char *str;
  int i;
  double d;
  fast_thc_type_t type;

  fast_thash_init(&fast_config);
  fast_thash_register_str(&fast_config,"ldap_server:0_to_1_host");
  fast_thash_register_str(&fast_config,"ldap_binddn:0_to_1_string");
  fast_thash_register_str(&fast_config,"ldap_use:0_to_1_int");
  fast_thash_set_int(fast_config,"ldap_use",1);
  fast_thash_register_str(&fast_config,"ldap_debug:0_to_1_int");
  fast_thash_set_int(fast_config,"ldap_debug",0);

  fast_thash_register_str(&fast_config,"nws_nameserver:0_to_1_host");
  fast_thash_register_str(&fast_config,"nws_forecaster:0_to_1_host");
  fast_thash_register_str(&fast_config,"nws_raw:0_to_1_int");
  fast_thash_set_int(fast_config,"nws_raw",0);
  fast_thash_register_str(&fast_config,"nws_use:0_to_1_int");
  fast_thash_set_int(fast_config,"nws_use",1);

  fast_thash_register_str(&fast_config,"autofetch_use:0_to_1_int");
  fast_thash_set_int(fast_config,"autofetch_use",1);
  fast_thash_register_str(&fast_config,"autofetch_on:0_to_1_double");
  fast_thash_set_double(fast_config,"autofetch_on",60);
  fast_thash_register_str(&fast_config,"autofetch_off:0_to_1_double");
  fast_thash_set_double(fast_config,"autofetch_off",600);


  /* we have to set the first one manually since stdarg.h does not allow us
     to declare all our arguments as variadic, and since the fast_thash_set_vargs
     function expects all its args in a va_list argument */
  if (!fast_thash_get_type(fast_config,first,&type))
    return 0;

  switch (type) {
  case fast_thc_host:
    str = va_arg(pa, char *);
    i=va_arg(pa,int);
    if (!fast_thash_set_host(fast_config,first,str,i))
      return 0;
    break;
    
  case fast_thc_string:
    str=va_arg(pa, char *);
    if (!fast_thash_set_string(fast_config, first, str))
      return 0;
    break;
    
  case fast_thc_int:
    i=va_arg(pa,int);
    if (!fast_thash_set_int(fast_config,first,i))
      return 0;
    break;

  case fast_thc_double:
    d=va_arg(pa,double);
    if (!fast_thash_set_double(fast_config,first,d))
      return 0; 
    break;
    
  default: 
    FAIL1("Option %s not valid.",first);
  }

  fast_thash_set_vargs(fast_config,pa);

  return fast_config_check();
}

/**
 *  fast_config_check:
 *  @Returns: True if everything went ok, false if an error occurs
 *
 *  Check that the passed options are valid
 */

int fast_config_check() {
  int use;
  char **strings,**hosts;
  int *ports;
  int size;

  if (!fast_thash_check(fast_config)) {
    FAIL("Error in fast configuration\n");
  }

  fast_thash_getfirst_int(fast_config,"ldap_use",&use);
  if (use) {
    fast_thash_get_host(fast_config,"ldap_server",&hosts,&ports,&size);
    if (!size) FAIL("If you want to use LDAP, you have to give me the server to use as argument.\n");

    fast_thash_get_string(fast_config,"ldap_binddn",&strings,&size);
    if (!size) FAIL("If you want to use LDAP, you have to give me the binddn to use as argument.\n");
  }

  fast_thash_getfirst_int(fast_config,"nws_use",&use);
  if (use) {
    fast_thash_get_host(fast_config,"nws_nameserver",&hosts,&ports,&size);
    if (!size) FAIL("If you want to use NWS, you have to give me the nameserver to use as argument.\n");

  }

  return 1;
}
