#include "fast_internals.h"

/*====[ prototypes ]=========================================================*/
int testLocal(int resource,const char *name);
int testNet(int resource,const char *name);

#define SENSOR_1  "localhost:9051"
#define SENSOR_2  "localhost:9053"

/*====[ main functions ]=====================================================*/
int main()
{
  int i;

  /* Redirect the outputs */
//  DirectDiagnostics(DIAGDEBUG, stdout);
//  DirectDiagnostics(DIAGINFO, stdout);
  DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stdout);
  DirectDiagnostics(DIAGERROR, stdout);
  DirectDiagnostics(DIAGFATAL, stdout);

  if (!fast_init("nws_use", 0,
		 "ldap_use",0,
		 NULL)) 
    return 1;

  printf ("Test each local resource 5 times, doing one test every 5 sec.\n");
  for (i=0; i<5; i++) {
    if (!testLocal(FAST_RESOURCE_AVAILABLE_CPU,"AVAILABLE CPU") || 
	!testLocal(FAST_RESOURCE_CURRENT_CPU,"CURRENT CPU")) {
      fflush(stdout);fflush(stderr);
      return 1;
    }

    printf("Wait 5 sec\n");
    sleep(5);
  }

    if (testLocal(FAST_RESOURCE_MEMORY,"MEMORY")) {
      FAIL("Interogation for memory resource using internal sensors shouldn't succeed\n");
    }
   if (testNet(FAST_RESOURCE_LATENCY,"LATENCY")) {
      FAIL("Interogation for latency resource using internal sensors shouldn't succeed\n");
   }

   LOG("Everything went ok (MEMORY and LATENCY tests are supposed to fail with internal sensor)\n");
  return 0;
}

int testLocal(int resource,const char *name) {
  double value;

  if (!fast_avail(resource,SENSOR_1,NULL,&value)){
    FAIL2("Can't retrive the %s from %s\n",name,SENSOR_1);
  } else {
    LOG3("%s(%s)=%f\n",name,SENSOR_1,value);
  }

  return 1;
}

int testNet(int resource,const char *name) {
  double value;

  if (!fast_avail(resource,SENSOR_1,SENSOR_2,&value)){
    FAIL3("Can't retrive the %s between %s and %s.\n",name,SENSOR_1,SENSOR_2);
  } else {
    LOG4("%s(%s ; %s)=%f\n",name,SENSOR_1,SENSOR_2,value);
  }

  return 1;
}
