#include "fast_internals.h"

#define SENSOR_1  "localhost:9051"
#define SENSOR_2  "localhost:9053"
#define NS   "localhost"
#define FORE "localhost"
#define NSPORT   9056
#define FOREPORT 9054
/*====[ prototypes ]=========================================================*/
extern int fast_nws_edges_list(int resource,
			       /* OUT */
			       int *vertex_count, char ***vertex_list,
			       int *edges_count, int **edges_list);

int testLocal(int resource,const char *name);
int testNet(int resource,const char *name);


/*====[ main functions ]=====================================================*/
int main()
{
  int i;

  /* Redirect the outputs */
  // DirectDiagnostics(DIAGDEBUG, stdout);
  DirectDiagnostics(DIAGINFO, stdout);
  DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stdout);
  DirectDiagnostics(DIAGERROR, stdout);
  DirectDiagnostics(DIAGFATAL, stdout);

  if (system("./mininws start") ||
      system("./mininws test")) {
    ERROR("Can't start mininws.\n");
    return 1;
  }

  if (!fast_init("nws_nameserver",NS,NSPORT,
		 "nws_raw",1,
		 "ldap_use",0,
		 NULL)) 
    return 1;

  printf ("Test each local resource 5 times, doing one test every 5 sec.\n");
  for (i=0; i<5; i++) {
    if (!testLocal(FAST_RESOURCE_AVAILABLE_CPU,"AVAILABLE CPU") || 
	!testLocal(FAST_RESOURCE_CURRENT_CPU,"CURRENT CPU") || 
	!testLocal(FAST_RESOURCE_MEMORY,"MEMORY")) {
      fflush(stdout);fflush(stderr);
      return 1;
    }
    printf("Wait 5 sec\n");
    sleep(5);
  }

  printf ("Test each network resource 5 times, doing one test every 10 sec.\n");
  for (i=0; i<5; i++) {
    if (!testNet(FAST_RESOURCE_LATENCY,"LATENCY") || 
	!testNet(FAST_RESOURCE_BANDWIDTH,"BANDWIDTH")) {
      fflush(stdout);fflush(stderr);
      return 1;
    }
    printf("Wait 30 sec\n");
    sleep(30);
  }

  if (system("./mininws stop")) {
    ERROR("Can't stop mininws\n");
    return 1;
  }

  return 0;
}

int testLocal(int resource,const char *name) {
  double value;

  if (!fast_avail(resource,SENSOR_1,NULL,&value)){
    FAIL2("Can't retrive the %s from %s\n",name,SENSOR_1);
  } else {
    LOG3("%s(%s)=%f\n",name,SENSOR_1,value);
  }

  return 1;
}

int testNet(int resource,const char *name) {
  double value;

  if (!fast_avail(resource,SENSOR_1,SENSOR_2,&value)){
    FAIL3("Can't retrive the %s between %s and %s.\n",name,SENSOR_1,SENSOR_2);
  } else {
    LOG4("%s(%s ; %s)=%f\n",name,SENSOR_1,SENSOR_2,value);
  }

  return 1;
}
