#include "fast_internals.h"
#include <time.h>

#define NB_ELM 200000

static int traverse(fast_cache_t *head) {
  fast_cache_cursor_t *cursor;
  char *key;
  int res;
  void *data;

  //  fast_cache_dump(head,(void (*)(void*))&printf);
  if (!(res=fast_cache_cursor_new(head,&cursor)))
    return res;

  printf("   Seen:");
  do {
    data=fast_cache_cursor_get_data(cursor);
    if (!(res=fast_cache_cursor_get_key(cursor,&key)))
      return head ? res : 1;
    printf (" [%s]",key);
    free(key);
  } while (fast_cache_cursor_next(cursor));
  printf ("\n");
  return 1;
}

static int countelems(fast_cache_t *head,int*count) {
  fast_cache_cursor_t *cursor;
  char *key;
  int res;
  void *data;
  *count=0;

  if (!(res=fast_cache_cursor_new(head,&cursor)))
    return res;

  do {
    data=fast_cache_cursor_get_data(cursor);
    if (!(res=fast_cache_cursor_get_key(cursor,&key)))
      return head ? res : 1;
    free(key);
    (*count)++;
  } while (fast_cache_cursor_next(cursor));
  return 1;
}

int main() {
  fast_cache_t *head=NULL;
  int i,j,k, nb;
  char *key;
  void *data;

  srand((unsigned int)time(NULL));

  printf("\nGeneric cache: CRASH test:\n");
  printf(" Fill the struct, count its elems and frees the structure (x20)\n");
  printf(" using 1000 elements with 1024 chars long randomized keys.\n");
  printf(" (a point is a test)\n");

  for (i=0;i<20;i++) {
    if (i%10) printf("."); else printf("%d",i/10); fflush(stdout);
    nb=0;
    for (j=0;j<1000;j++) {
      if (!(key=malloc(1024))) {
	fprintf(stderr,"Out of memory\n");
	return 1;
      }

      for (k=0;k<1023;k++)
	key[k]=rand() % ('z' - 'a') + 'a';
      key[k]='\0';
      if (!fast_cache_insert(&head,strdup(key),key,&free))
	return 1;
    }
    nb=0;
    if (!countelems(head,&nb) || nb != 1000) {
       printf ("\nI found %d elements, and not 1000\n",nb);
       return 1;
    }	  
    if (!fast_cache_free(&head))
      return 1;
  }


  printf("\n Fill 200 000 elements, with keys being the number of element\n");
  printf("  (a point are 10 000 elements)\n");
  for (j=0;j<NB_ELM;j++) {
    if (!(j%10000)) printf("."); fflush(stdout);
    if (!(key=malloc(10))) {
      fprintf(stderr,"Out of memory\n");
      return 2;
    }
    
    sprintf(key,"%d",j);
    if (!fast_cache_insert(&head,strdup(key),key,&free))
      return 2;
  }

  printf("\n Count the elements (retrieving the key and data for each): \n");
  if (!countelems(head,&i)) 
    return 1;
  printf(" There is %d elements\n",i);
  printf("\n Search my 200 000 elements 20 times. (a point is a test)\n");
  if (!(key=malloc(10))) {
    fprintf(stderr,"Out of memory\n");
    return 3;
  } 
  for (i=0;i<20;i++) {
    if (i%10) printf("."); else printf("%d",i/10); fflush(stdout);
    for (j=0;j<NB_ELM;j++) {
      
      sprintf(key,"%d",j);
      if (!fast_cache_retrieve(head,key,&data) ||
	  strcmp(key,(char*)data)) {
	printf("key=%s != data=%s\n",key,(char*)data);
	return 2;
      }
    }
  }

  printf("\n Remove my 200 000 elements. (a point are 10 000 elements)\n");
  if (!(key=malloc(10))) {
    fprintf(stderr,"Out of memory\n");
    return 3;
  }
  for (j=0;j<NB_ELM;j++) {
    if (!(j%10000)) printf("."); fflush(stdout);
    
    sprintf(key,"%d",j);
    if (!fast_cache_remove(head,key)) {
      printf("remove key=%s failed!\n",key);
      //      fast_cache_dump(head,(void (*)(void*))printf);
      return 3;
    }
  }

  if (!fast_cache_free(&head))
    return 4;
  
  if (!fast_cache_free(&head))
    return 4;
  printf("\n");
  return 0;
}
