#include "fast_internals.h"

static int fill(fast_cache_t **head);
static int debuged_add(fast_cache_t **head,const char*key);
static int search(fast_cache_t *head,const char*key);
static int debuged_remove(fast_cache_t *head,const char*key);
static int traverse(fast_cache_t *head);

static int fill(fast_cache_t **head) 
{
  printf(" Fill in the struct\n");

  if (// Fill in the struct
      !debuged_add(head,"12")     ||
      !debuged_add(head,"12a")    ||
      !debuged_add(head,"12b")    ||
      !debuged_add(head,"123")    ||
      !debuged_add(head,"123456") ||
      // Child becomes child of what to add
      !debuged_add(head,"1234")    ||
      // Need of common ancestor
      !debuged_add(head,"123457"))
    return 0;
  
  return 1;
}

static int debuged_add(fast_cache_t **head,const char*key)
{
  int res;
  LOG1("Add %s\n",key);
  res=fast_cache_insert(head,strdup(key),strdup(key),&free);
  //  fast_cache_dump(*head,(void (*)(void*))&printf);
  return res;
}

static int search(fast_cache_t *head,const char*key)
{
  void *data;
  int res;
  res=fast_cache_retrieve(head,key,&data);
  //  printf("%s\n",data?(char*)data:"(null)");
  return (res && !strcmp((char*)data,key));
}

static int debuged_remove(fast_cache_t *head,const char*key)
{
  int res;
  LOG1("Remove %s\n",key);fflush(stdout);
  res=fast_cache_remove(head,key);
  //  fast_cache_dump(head,(void (*)(void*))&printf);
  return res;
}

static int traverse(fast_cache_t *head) {
  fast_cache_cursor_t *cursor=NULL;
  char *key;
  int res;

  //  fast_cache_dump(head,(void (*)(void*))&printf);
  if (!(res=fast_cache_cursor_new(head,&cursor)))
    return res;

  //  printf("   Seen:");
  do {
    if (!(res=fast_cache_cursor_get_key(cursor,&key)))
      return head ? res : fast_cache_cursor_free(cursor),1;
    //    printf (" [%s]\n",key);
    free(key);
  } while (fast_cache_cursor_next(cursor));
  //  printf ("\n");
  fast_cache_cursor_free(cursor);
  return 1;
}

int main() {
  fast_cache_t *head=NULL;
  char *data;

  //DirectDiagnostics(DIAGDEBUG, stdout);
  DirectDiagnostics(DIAGINFO, stdout);
  //DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stderr);
  DirectDiagnostics(DIAGERROR, stderr);
  DirectDiagnostics(DIAGFATAL, stderr);

  printf("\nGeneric cache: USAGE test:\n");

  printf(" Traverse the empty cache\n");
  if (!traverse(head)) return 1;

  if (!fill(&head)) return 1;

  printf(" Change some values\n");
  LOG("Change 123 to 'Changed 123'\n");
  if (!fast_cache_insert(&head,strdup("123"),strdup("Changed 123"),&free))
    return 2;
  //  fast_cache_dump(head,(void (*)(void*))&printf);
  printf(" Traverse the resulting cache\n");
  if (!traverse(head)) return 1;

  printf(" Retrive values\n");
  if (!fast_cache_retrieve(head,"123",(void**)&data) || !data ||
      strcmp("Changed 123",data) ||

      !fast_cache_retrieve(head,"Can't be found",(void**)&data) || data ||
      !fast_cache_retrieve(head,"123 Can't be found",(void**)&data) || data ||
      !fast_cache_retrieve(head,"12345678 NOT",(void**)&data) || data ||

      !search(head,"12a")    ||
      !search(head,"12b")    ||
      !search(head,"12")     ||
      !search(head,"123456") ||
      !search(head,"1234")    ||
      !search(head,"123457"))
    return 3;
  printf(" Traverse the resulting cache\n");
  if (!traverse(head)) return 1;

  //  fast_cache_dump(head,(void (*)(void*))&printf);

  printf(" Free the struct\n");
  if (!fast_cache_free(&head) ||
      !fast_cache_free(&head)) // frees it twice to see if it work
    return 4;
  printf(" Traverse the resulting cache\n");
  if (!traverse(head)) return 1;

  printf("\n");
  if (!fill(&head)) return 1;
  printf(" Remove the data (traversing the resulting cache each time)\n");
  if (debuged_remove(head,"Does not exist") ||
      !traverse(head) ||
      debuged_remove(head,"12345") || /* internal entry */
      !traverse(head) ||
      !debuged_remove(head,"12a") ||
      !traverse(head) ||
      !debuged_remove(head,"12b") ||
      !traverse(head) ||
      !debuged_remove(head,"12") ||
      !traverse(head) ||
      !debuged_remove(head,"123456") ||
      !traverse(head) ||
      !debuged_add(&head,"12346") ||
      !traverse(head) ||
      !debuged_remove(head,"1234") ||
      !traverse(head) ||
      !debuged_remove(head,"123457") ||
      !traverse(head) ||
      !debuged_remove(head,"123") ||
      !traverse(head) ||
      !debuged_remove(head,"12346") ||
      !traverse(head))
    return 5;
  
  if (!fast_cache_free(&head) ||
      !fast_cache_free(&head)) // frees it twice to see if it work
    return 4;

  return 0;
}
