#include "fast_internals.h"

#define NB_ELM 1000
#define MULTICACHE_DEPTH 5
#define KEY_SIZE 512
#define NB_TEST 20

static int test1();

static int test1() {
  fast_cache_t *head=NULL;
  int i,j,k,l;
  char **key = NULL;
  char **val = NULL;
  void *data;

  printf("\nGeneric multicache: CRASH test:\n");
  printf(" Fill the struct and frees it %d times, using %d elements, depth of multicache=%d\n",NB_TEST,NB_ELM,MULTICACHE_DEPTH);
  printf(" with %d chars long randomized keys. (a point is a test)\n",KEY_SIZE);

  for (i=0;i<NB_TEST;i++) {
    if (i%10) printf("."); else printf("%d",i/10); fflush(stdout);
    if (!(key=malloc(sizeof(char*)*MULTICACHE_DEPTH)))
      FAIL("Out of memory\n");      
    if (!(val=malloc(sizeof(char*)*MULTICACHE_DEPTH)))
      FAIL("Out of memory\n");      
    for (l=0 ; l<MULTICACHE_DEPTH ; l++)
      if (!(val[l]=malloc(sizeof(char)*KEY_SIZE)))
	FAIL("Out of memory\n");

    for (j=0;j<NB_ELM;j++) {
//      printf ("Add ");
      for (l=0 ; l<MULTICACHE_DEPTH ; l++){
	for (k=0;k<KEY_SIZE-1;k++) {
	  val[l][k]=rand() % ('z' - 'a') + 'a';
	}
	val[l][k]='\0';
//printf("%s ; ",val[l]);
	key[l]=strdup(val[l]);
      }
//printf("in multitree %p.\n",head);
      if (!fast_multicache_insert(&head,MULTICACHE_DEPTH,key,
				  strdup(val[0]),&free))
	  return 0;
//   fast_cache_dump(head,&fast_cache_print);
      if (!fast_multicache_retrieve(head,
				    MULTICACHE_DEPTH,(const char **)val,
				    &data)) {
	return 0;
      } else if (!data || strcmp((char*)data,val[0])) {
	FAIL2("Retrieved value (%s) does not match the entrered one (%s)\n",
	      (char*)data,val[0]);
      }
    }
    if (!fast_cache_free(&head))
      return 0;

    for (l=0 ; l<MULTICACHE_DEPTH ; l++)
      if (val[l]) free(val[l]);
    free(val);
    free(key);

  }

  printf("\n");
  return 1;
}

int main() {
  //DirectDiagnostics(DIAGDEBUG, stdout);
  DirectDiagnostics(DIAGINFO, stdout);
  //  DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stderr);
  DirectDiagnostics(DIAGERROR, stderr);
  DirectDiagnostics(DIAGFATAL, stderr);

  return !(test1());
}
