/* $Id: test_thash.c,v 1.1 2003/06/03 14:04:03 mquinson Exp $ */

/* test_options - test code to the options set */

#include "fast_internals.h"

/*====[ Prototypes ]=========================================================*/
fast_thash_t *make_set(void); /* build a minimal set */
int test1(void); /* set creation test */
int test2(void); /* unregistred option */
int test3(void); /* validate=>not enought */
int test4(void); /* validate=> too many */
int test5(void); /* get users list */


/*====[ Code ]===============================================================*/
fast_thash_t *make_set(){
  fast_thash_t *set=NULL; 

  fast_thash_register_str(&set,"hostname:1_to_1_string");
  fast_thash_register_str(&set,"user:1_to_10_string");
  fast_thash_register_str(&set,"speed:1_to_1_int");

  fast_thash_set_parse(set,
		   "hostname:veloce "
		   "user:mquinson\nuser:ecaron\tuser:fsuter");
  return set;
}

/*----[ set creation test ]--------------------------------------------------*/
int test1() 
{
  fast_thash_t *set=make_set();
  fast_thash_free(&set);
  return 1;
}

/*----[ unregistred option ]-------------------------------------------------*/
int test2() 
{
  fast_thash_t *set=make_set();
  fast_thash_set_parse(set,"color:blue");
  fast_thash_free(&set);
  return 1;
}

/*----[ validate=>not enought ]----------------------------------------------*/
int test3() 
{
  fast_thash_t *set=make_set();
  fast_thash_check(set);
  fast_thash_free(&set);
  return 1;
}

/*----[ validate=> too many ]------------------------------------------------*/
int test4() 
{
  int val;
  fast_thash_t *set=make_set();
  fast_thash_set_parse(set,"speed:42 speed:24");
  fast_thash_check(set);
  fast_thash_getfirst_int(set,"speed",&val);
  printf("speed value: %d\n",val);
  fast_thash_free(&set);
  return 1;
}

/*----[ get users list ]-----------------------------------------------------*/
int test5()
{
  char **options;
  int optCount;
  int i;

  fast_thash_t *set=make_set();
  fast_thash_set_parse(set,"speed:42");
  fast_thash_check(set);
  fast_thash_get_string(set,"user",&options,&optCount);
  printf("Count: %d; Options: \n",optCount);
  for (i=0;i<optCount;i++)
    printf("%s\n",options[i]);
  fast_thash_free(&set);
  return 1;
}
 
int main() 
{
  //DirectDiagnostics(DIAGDEBUG, stdout);
  DirectDiagnostics(DIAGINFO, stdout);
  DirectDiagnostics(DIAGLOG, stdout);
  DirectDiagnostics(DIAGWARN, stderr);
  DirectDiagnostics(DIAGERROR, stderr);
  DirectDiagnostics(DIAGFATAL, stderr);

  printf(" TEST1 \n Expected : No output\n");
  test1();

  printf("\n %s \n Expected: %s\n",
	 "TEST2",
	 "No registrated cell corresponding to 'color'.");
  test2();

  printf("\n %s \n Expected: %s\n",
	 "TEST3",
	 "Cell type speed needs at least 1 int, but there is only 0 values.");
  test3();

  printf("\n %s \n Expected: %s\n",
	 "TEST4",
	 "speed value: 24");
  test4();

  printf("\n %s \n Expected: %s\n",
	 "TEST5",
	 "Count: 3; Options:\\nmquinson\\necaron\\nfsuter");
  test5();

  return 0;
}

