/* $Id: cache.h,v 1.1 2003/05/26 10:51:54 mquinson Exp $ */

/* cache.h -- api to a generic cache system                                  */

/* Copyright (C) 2001 the OURAGAN project. Code GPL'ed (see www.gnu.org).    */
/* Author(s): Martin Quinson                                                 */

#ifndef _FAST_CACHE_H
#define _FAST_CACHE_H

#ifdef  __cplusplus
extern "C" 
#endif

/*####[ Type definition ]####################################################*/
typedef struct fast_cache_ fast_cache_t;

/*####[ Simple cache functions ]#############################################*/

/*----[ fast_cache_free ]----------------------------------------------------*/
/* Frees a cache structure with all its childs.                              */
/*---------------------------------------------------------------------------*/
int fast_cache_free(fast_cache_t **cache);

/*----[ fast_cache_insert ]--------------------------------------------------*/
/* Insert the data in the structure under the #key#                          */
/* The key is destroyed in the process. Think to strdup it before.           */
/* Returns if it was ok or not                                               */
/*---------------------------------------------------------------------------*/
int fast_cache_insert(fast_cache_t **head,
		      char *key,
		      void *data,void (*free_ctn)(void*));

/*----[ fast_cache_retrieve ]------------------------------------------------*/
/* Search the given #key#. data=NULL when not found.                         */
/* Returns true if anything went ok, and false on internal error.            */
/*---------------------------------------------------------------------------*/
int fast_cache_retrieve(fast_cache_t *head,const char *key,
			/* OUT */void **data);
/*----[ fast_cache_remove ]--------------------------------------------------*/
/* Remove the entry associated with the given #key#.                         */
/* Returns if ok. Removing a non-existant key is ok.                         */
/*---------------------------------------------------------------------------*/
int fast_cache_remove(fast_cache_t *head,const char *key);

/*----[ fast_cache_dump ]----------------------------------------------------*/
/* Outputs the content of the structure. (for debuging purpose)              */
/* #output# is a function to output the data.If NULL, data won't be displayed*/
/* Returns if it was ok or not                                               */
/*---------------------------------------------------------------------------*/
int fast_cache_dump(fast_cache_t *head,
		    void (*output)(void*));

/*----[ fast_cache_print ]---------------------------------------------------*/
/* To dump multicache, this function dump a cache                            */
/*---------------------------------------------------------------------------*/
void fast_cache_print(void *data);
/* To dump multicache, this one dumps a string                               */
void fast_cache_prints(void *data);


/*####[ Multi cache functions ]##############################################*/
/* The are cache of cache of data. Any function there works the same way     */
/*  than their simple cache counterpart.                                     */
/*###############################"###########################################*/

/*----[ fast_multicache_free ]-----------------------------------------------*/
/* This function does not exist. Use fast_cache_free instead.                */
/*---------------------------------------------------------------------------*/

/*----[ fast_multicache_insert ]---------------------------------------------*/
/* Insert the data in the structure under the #keycount# #key#s.             */
/* The key are destroyed in the process. Think to strdup it before.          */
/* Returns if it was ok or not                                               */
/*---------------------------------------------------------------------------*/
int fast_multicache_insert(fast_cache_t **head,
			   int keycount,char **key,
			   void *data,void (*free_ctn)(void*));

/*----[ fast_multicache_retrieve ]-------------------------------------------*/
/* Search the given #key#. data=NULL when not found.                         */
/* Returns true if anything went ok, and false on internal error.            */
/*---------------------------------------------------------------------------*/
int fast_multicache_retrieve(fast_cache_t *head,
			     int keycount,const char **key,
			     /* OUT */void **data);

/*----[ fast_multicache_remove ]---------------------------------------------*/
/* Remove the entry associated with the given #key#.                         */
/* Returns if ok. Removing a non-existant key is ok.                         */
/*---------------------------------------------------------------------------*/
int fast_multicache_remove(fast_cache_t *head,
			   int keycount,const char **key);

/*####[ Cache cursor functions ]#############################################*/
/* To traverse (simple) caches                                               */
/* Don't add or remove entries to the cache while traversing !!!             */
/*###########################################################################*/
typedef struct fast_cache_cursor_ fast_cache_cursor_t;
/* creator/destructor */
int fast_cache_cursor_new(fast_cache_t *head,
			  /*OUT*/fast_cache_cursor_t **cursor);
void fast_cache_cursor_free(fast_cache_cursor_t *cursor);

/* next element. Returns true if ok, and false on last entry */
int fast_cache_cursor_next(fast_cache_cursor_t *cursor);
/* back to first element 
   it is not enough to reinit the cache after an add/remove in cache*/
int fast_cache_cursor_rewind(fast_cache_cursor_t *cursor);

/* Get current element. The key will be overwriten over calls */
int fast_cache_cursor_get_key(fast_cache_cursor_t *cursor,
			      /*OUT*/char **key);
void *fast_cache_cursor_get_data(fast_cache_cursor_t *cursor);

#ifdef  __cplusplus
}
#endif

#endif /* _FAST_CACHE_H */
