/* $Id: avail.h,v 1.3 2003/06/10 13:36:11 pmachard Exp $ */

/* avail.h - Low level interface to system's availability forecasting.    */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                      */
/* Code available under BSD-like licence (see COPYRIGHT file).            */
/* Author(s): Martin Quinson                                              */

#ifndef _FAST_AVAIL_H_
#define _FAST_AVAIL_H_

#ifdef __cplusplus
extern "C" {
#endif

/*----[ available dynamical resources enum ]---------------------------------*/
#define FAST_RESOURCE_TYPE_COUNT    7

#define FAST_RESOURCE_AVAILABLE_CPU 0
#define FAST_RESOURCE_CURRENT_CPU   1
#define FAST_RESOURCE_MEMORY        2
#define FAST_RESOURCE_DISK          3
#define FAST_RESOURCE_BANDWIDTH     4
#define FAST_RESOURCE_LATENCY       5
#define FAST_RESOURCE_CPU_COUNT     6

/*----[ fast_avail ]---------------------------------------------------------*/
/* Retrieve a #value# from the NWS system of type #resource_type# (one of the*/
/*  FAST_RESOURCE_*).                                                        */
/* If there is a resource concerning a unique host (as CPU or Memory),       */
/*  #hostSrc# is its name (preferably the short name, without the domainname)*/
/*  of this host and #hostDest# is ignored.                                  */
/* If there is a resource concerning two hosts, #hostSrc# and #hostDest# are */
/*  their names. Network resources are not symetric, ie BW(A,B) != BW(B,A)   */
/*                                                                           */
/* The function returns true if anything went ok and false if not.           */
/*---------------------------------------------------------------------------*/
int fast_avail(int resource,const char *hostSrc,const char *hostDest,
		   /* OUT */ double *value);

/*====[ Simple interfacing wrapper ]=========================================*/
/*                                                                           */
/* Get the status of the system in case you want to redo the computation,    */
/*  or if you do a visualisation tool. (hello Eddy ;)                        */
/*                                                                           */
/* The input data are always the name of the machines, for example "pixies". */
/*  it could be the long name (ie "pixies.ens-lyon.fr"), but it's safer to   */
/*  use the short name (I build a request for nws with (host=#hostname#*),   */
/*  so the '*' will match the long name if you give the short one, but not   */
/*  the contrary.                                                            */
/*                                                                           */
/* Here are the units used:                                                  */
/*   load: free CPU fraction                                                 */
/*   memory: Megabytes                                                       */
/*   latency: Milliseconds                                                   */
/*   bandwidth: Megabits/second                                              */
/*                                                                           */
/* All these functions return true if all went ok, and false on error.       */
/*                                                                           */
/*===========================================================================*/

int fast_avail_load(const char *hostName,double *value);
int fast_avail_cpucount(const char *hostName,int *value);
int fast_avail_memory(const char *hostName,double *value);
int fast_avail_latency(const char *source,const char *dest,double *value);
int fast_avail_bandwidth(const char *source,const char *dest,double *value);

/* Dont call it directly unless you know what you're doing. booking will do 
   it for you */
void fast_avail_reset_cpu_measurements(const char *host);


#ifdef __cplusplus
}
#endif 
#endif /* _FAST_AVAIL_H_ */
