/* $Id: data_arg.h,v 1.6 2003/06/10 14:52:54 mquinson Exp $ */

/* data_arg - How to describe a routine argument                          */

/* Copyright (C) 2003 the OURAGAN project.                                */
/* Code available under BSD-like licence (see COPYRIGHT file).            */
/* Author(s): Martin Quinson                                              */

#ifndef _DATA_ARG_H
#define _DATA_ARG_H

#include <sys/types.h>

/* C++ nuisance */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * data_arg_t: structure describing any needed information about one
 * argument of a routine to be scheduled. This is an opaque structure which
 * should only be accessed through the dedicated functions 
 */
typedef struct data_arg_s data_arg_t;

/**
 * data_scal_type_t: enum on the possible scalar types
 */

typedef enum {
  data_char = 0, data_int, data_double,
  data_scal_type_count /* Sentinel. Must be the last one */
} data_scal_type_t;

/** data_cons_t: enum on the different constructor of complex structures */
typedef enum {
  data_cons_scal  = 0, data_cons_vect, data_cons_mat, data_cons_file,
  data_cons_type_count /* Sentinel. Must be the last one */
} data_cons_type_t;


/****[ Malloc ]****************************************************************/
data_arg_t *data_arg_new(data_scal_type_t scal, data_cons_type_t cons, ...);

void data_arg_free(data_arg_t **arg);
int data_arg_check(data_arg_t *arg); /* checks that min / max are respected */
void data_arg_dump(data_arg_t *da,const char *name,const char *indent); /* to debug */

/* simple types */
data_arg_t *data_arg_char_new(char value);
data_arg_t *data_arg_int_new(int value);
data_arg_t *data_arg_double_new(double value);

/* constructed types */
data_arg_t *data_arg_vect_new(data_scal_type_t type,size_t size);
data_arg_t *data_arg_string_new(const char *val);

data_arg_t *data_arg_mat_new(data_scal_type_t type, 
			     size_t rows, size_t cols);

data_arg_t *data_arg_file_new(const char *path);

/****[ Set properties in place ]**********************************************/
/* Existing properties are: 
     - Scalars: value:1_to_1_<type of this scalar>

     - Char vector (=string): value:0_to_1_string

     - Vectors: size:1_to_1_int
     - Matrices: rows:1_to_1_int cols:1_to_1_int row_major:0_to_1_int
     - Files: size:0_to_1_int path:1_to_1_string */

int data_arg_pset_int   (data_arg_t *da, const char *key, int val);
int data_arg_pset_double(data_arg_t *da, const char *key, double val);
int data_arg_pset_string(data_arg_t *da, const char *key, const char *val);


data_scal_type_t data_arg_scaltype_get(data_arg_t *da);
data_cons_type_t data_arg_constype_get(data_arg_t *da);

#ifdef __cplusplus
}
#endif

#endif /* _DATA_ARG_H */
