/* $Id: data_argstack.h,v 1.4 2003/06/06 14:59:22 pmachard Exp $ */

/* data_argstack - How to describe the set of arguments a routine expects */

/* We want to stay here as close as reasonable to the GridRPC 
   recommendation, which is far too imprecise to be implementable.        */

/* Copyright (C) 2001 the OURAGAN project.                                */
/* Code available under BSD-like licence (see COPYRIGHT file).            */
/* Author(s): Martin Quinson                                              */

#ifndef _DATA_ARGSTACK_H
#define _DATA_ARGSTACK_H

#include <sys/types.h>

/* C++ nuisance */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * data_arg_stack_t:
 *
 * Description of all the arguments passed to a routine to schedule. 
 * It may be seen as a run-time va_args stuff.
 */
typedef struct data_argstack_s data_argstack_t;


/* ---- [ memory management ] ---- */
data_argstack_t *data_argstack_new(const char *routine_name);
void data_argstack_free(data_argstack_t *as);
int data_argstack_check(data_argstack_t *as);

/* ---- [ High level creation ] ---- */
int data_argstack_create(data_argstack_t **as, const char *routine_name, int argcount,...);


/* ---- [ Access ] ---- */
int data_argstack_push(data_argstack_t *as, data_arg_t *arg);
data_arg_t *data_argstack_pop(data_argstack_t *as);
data_arg_t *data_argstack_get(data_argstack_t *as, int pos);

char *data_argstack_routine_get(data_argstack_t *as);

int data_argstack_split(data_argstack_t *as, char **strings, 
			double **nums, int *num_used);

/* ---- [ Debug ] ---- */
void data_argstack_dump(data_argstack_t *as);

/* C++ nuisance */
#ifdef __cplusplus
}
#endif

#endif /*_DATADESCRIPTION_H */
