/* $Id: high_level.h,v 1.3 2003/06/14 09:16:28 mquinson Exp $ */

/* high_level.h - High level interface to FAST.                           */

/* Copyright (C) 2001,2002,2003 the OURAGAN project.                      */
/* Code available under BSD-like licence (see COPYRIGHT file).            */
/* Author(s): Martin Quinson                                              */

#ifndef _FAST_HIGH_LEVEL_H_
#define _FAST_HIGH_LEVEL_H_

#ifdef __cplusplus
extern "C" {
#endif


/*----[ fast_init ]-------------------------------------------------------*/
/* Push some option settings to fast.                                     */
/*  Each option comes in several arguments: name and value                */
/* see in the code of fast_config_register to see the list of valid names */
/*  (or the tobewritten documentation ;)                                  */
/* Each value can be of type string, int, double or host                  */
/*  (host values are pairs of hostname (string) and port (integer))       */
/*                                                                        */
/* Last argument must be NULL (unless you want to segfault ;)             */
/*                                                                        */
/* Note that previously existing settings are not removed by this function*/
/* use fast_config_set if you want to *change* a value                    */

int fast_init(const char *first,...);

/* Check that the passed options are valid */
int fast_config_check(void);

/*====[ fast_comm_time ]==================================================*/
/*                                                                        */
/* Compute the needed time (in seconds) to send #howmany# bytes from the  */
/*   #source# to the #dest#.                                              */
/*                                                                        */
/* The result is a fast_comm_result_t structure, described below.         */
/*                                                                        */
/*....[ fast_comm_result_t ]..............................................*/
/*                                                                        */
/* One day, FAST will be multi protocol (for now, only TCP is available). */
/* This day, the result of fast_comm_time will be a #nb# long list of:    */
/*   {#protocol# ; #time# over this protocol}                             */
/* This list is sorted, the shortest time first.                          */
/*                                                                        */
/*========================================================================*/

struct fast_comm_result_struct {
  int nb;
  int *protocol; /* identificator of the protocol */
  double *time;
};
typedef struct fast_comm_result_struct fast_comm_result_t;
 
/*----[ fast_comm_result_free ]----------------------------------------------*/
void fast_comm_result_free(/*@only@*//*@null@*/fast_comm_result_t *res);

/*----[ fast_comm_time_list ]------------------------------------------------*/
int fast_comm_time_list(const char * source,const char * dest,size_t howbig,
		   /*@OUT@*/fast_comm_result_t **where);

/*----[ fast_comm_time_best ]------------------------------------------------*/
/* Returns the time of the best forecast (and -1 on error).                  */
int fast_comm_time_best(const char * source,const char * dest,size_t howbig,
			/* OUT */ double *value);

/*====[ fast_comp_time ]=====================================================*/
/*                                                                           */
/* Compute the needed time (in seconds) to execute #function# on #host#.     */
/*                                                                           */
/* FIXME: we need a result type as for comm time to give the configuration of*/
/*  the parallal task which can achive the given forecast, but I've so few   */
/*  clue about all this stuff that I prefer skip it for now...               */
/*                                                                           */
/*===========================================================================*/

int fast_comp_time_best(const char *host,data_argstack_t *function,
			/* OUT */ double *value);

#ifdef __cplusplus
}
#endif


#endif /* _FAST_HIGH_LEVEL_H_ */
