/* $Id: fast_internals.h,v 1.7 2005/03/05 17:49:21 mquinson Exp $ */

/* fast_internals -- stuff needed in several point of FAST, but not public */

/* Copyright (C) 2003 the OURAGAN project. Code GPL'ed (see www.gnu.org). */
/* Author(s): Martin Quinson                                              */

#ifndef FAST_INTERNALS_H
#define FAST_INTERNALS_H
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <string.h>

#include "../config.h"
#include "fast.h"
#include "Utils/cache.h"
#include "Utils/typed_hash.h"
#include "Utils/diagnostic.h"
#include "Estimator/estimator.h"

/* Let availability stuff use the bookings */
#include "Avail/fast_booking.h"
int fast_cbook_use(int resource,double measurement_time,const char *host,
		      double *value);

/* Initialize the submodules */
int fast_avail_init(void);
int fast_need_init(void);

#ifndef MAX
#define MAX(a,b) ((a)>(b) ? (a) : (b))
#endif
/*****************************************************************************/
/**** Configuration handling                                              ****/
/*****************************************************************************/
extern fast_thash_t *fast_config;

int fast_config_init(const char *first, va_list pa);
/*----[ Setting ]---------------------------------------------------------*/
/* string values are strdup'ed before use, so you have to free your copy  */

#define fast_config_set_int(name,val)    fast_thash_set_int   (fast_config,name,val)
#define fast_config_set_double(name,val) fast_thash_set_double(fast_config,name,val)
#define fast_config_set_string(name,val) fast_thash_set_string(fast_config,name,val)
#define fast_config_set_host(name,h,p)   fast_thash_set_host  (fast_config,name,h,p)

#define fast_config_rm_int(name,val)     fast_thash_rm_int    (fast_config,name,val)
#define fast_config_rm_double(name,val)  fast_thash_rm_double (fast_config,name,val)
#define fast_config_rm_string(name,val)  fast_thash_rm_string (fast_config,name,val)
#define fast_config_rm_host(name,h,p)    fast_thash_rm_host   (fast_config,name,h,p)

#define fast_config_empty(name)          fast_thash_empty(fast_config,name)
			
/*----[ Getting ]---------------------------------------------------------*/
/* Returns a pointer to the values actually stored in the cache. Do not   */
/* modify them unless you really know what you're doing.                  */
#define fast_config_get_int(name,val,size)    fast_thash_get_int   (fast_config,name,val,size)
#define fast_config_get_double(name,val,size) fast_thash_get_double(fast_config,name,val,size)
#define fast_config_get_string(name,val,size) fast_thash_get_string(fast_config,name,val,size)
#define fast_config_get_host(name,h,p,size)   fast_thash_get_host  (fast_config,name,h,p,size)

/* Get only the first entry. Usefull if you know there is only one elem */
#define fast_config_getfirst_int(name,val)    fast_thash_getfirst_int   (fast_config,name,val)
#define fast_config_getfirst_double(name,val) fast_thash_getfirst_double(fast_config,name,val)
#define fast_config_getfirst_string(name,val) fast_thash_getfirst_string(fast_config,name,val)
#define fast_config_getfirst_host(name,h,p)   fast_thash_getfirst_host  (fast_config,name,h,p)

#define ASSERT(test,msg)   \
  do {                     \
     if (test) ABORT(msg); \
  } while (0)

#endif /* FAST_INTERNALS_H */
